
# q-expansion of newform 96.12.a.d, downloaded from the LMFDB on 23 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 96
weight = 12
poly_data = [-486, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[-48, 96], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0], [-243, 0], [2650, -3], [19436, 25], [523700, 242], [-11450, -206], [4366650, -1606], [-3673300, 1650], [-3355872, -6350], [90090346, 27775], [-105790700, -52459], [56111350, 290444], [-363480590, -51650], [-108204428, 591750], [1087389544, -507450], [-56012350, -1883269], [1621974700, -3712824], [8263115310, -776800], [10386309556, 4441800], [10318550800, 5066234], [-9274101750, -1623688], [-14115041900, 1180441], [-3594641028, -24498550], [51839590394, 1179100], [99807243250, -4702276], [60213754962, -39079325], [21488104452, -5640025], [-46135308116, 66124300], [212678618390, -10140050], [167227785250, -55605956], [102577723036, -141780225], [269775086900, -40937392], [641262466050, -6169894], [430105519900, 11288816], [-20393344950, 41468125], [-874713183300, 469277609], [380530176350, -381458468], [1868195849132, -129972250], [-2037063740984, -393297800], [-1256883541950, 147496569], [1530815268900, -772133160], [-3545449711282, -161847450], [-1684833974400, -1576993384], [-836344195950, 855748712], [-2058348641150, -357958481], [3442707615700, 1197858227], [10635467903300, 485165048], [-133105110292, -2525481925], [-3073579718916, -2293978050], [1467266203694, 6086890650], [-6112026060150, -2067146868], [-12070381569600, 4665597116], [10978335532690, 2720312700], [1533976251300, -1128777742], [16819685416450, 7917415232], [-12457155040728, 7904852000], [15042966934170, -4276630625], [-4153402882500, -1220917773], [3424592680750, -6708383882], [27813222578730, 4301698400], [30170437926988, 415059200], [34074409972050, 9993739531], [29849907452196, 21007351400], [9648497965800, -22954057084], [-5190700281350, 20883231864], [4174202917050, 20040070307], [22309251575500, -43580062900], [-8892541913950, 57014145112], [-57727675524284, 44460109850], [50588203016014, -19872125600], [5630361633650, -23945657220], [-59294463221200, 28513751482], [210857373369924, 11163019725], [-137025315747850, -34568289432], [72251190588500, -10458961998], [-31811699332832, 6877694800], [-86787058465350, 70239924125], [-94257838710050, -35402888856], [43371457199498, 135181033950], [-165939213689990, -70068316700], [-115472173272100, -103544760490], [-110293255407010, 148589558150], [90970607715000, -201120967718], [120843716880850, 177696466936], [-137773361159500, -180795569409], [-380011618128348, -54676835350], [12497533703210, -269618773550], [239119381673450, -84521688500], [-272906026294782, -138880998975], [13488193659292, 241046498875], [201567516475916, -24758026250], [369780597429000, 19601773434], [-375432514716436, 229162341425], [628258386509900, -73560809092], [-642093077214300, 32096603232], [-884817043270368, -95445433550], [230085175622586, 466946465675], [448576676868882, -84421922250], [100646085882468, -596759156350], [1004743043829478, -84577547450], [479758578148284, -199491491750], [-1046498274755150, 151274275917], [111319327436028, 755429702250], [660948750591970, 204830008350], [-612589687569700, 816250665900], [-1506296646279150, 598668631576], [844230328764684, -1053355234300], [1222939902355650, -28301256004], [-469624911546800, -693099700418], [-502611897189830, 1089352296000], [-3816305070240356, 94445763575], [-2164999329265050, 300411199680], [1499564839769850, 497235502920], [2039160061107900, -783334311352], [-2395224428790500, 962885167609], [-800824456843990, -825192608950], [-3000962659078212, 645945898050], [-2324595951740976, -910714397950], [-2248389194167150, -91186777755], [126096029010900, -172284223308], [2692411032391910, 1029547187100], [-952757177113150, -1213937317136], [-868584351800950, -811445423967], [2802553818318372, -1940750989050], [-1986241758266500, 379646502982], [-3884033482073670, 399330643475], [185547657078286, 1137806369850], [4720766529149800, 42352927286], [-1513875862790724, -3069580305575], [4561802193644150, -437680982518], [-2352378354279100, -2702854388964], [-6366148073214312, -44665897700], [9279508666378900, 1284270803473], [997101971469150, -854318520582], [699542714150322, -4211452287350], [7230176745271666, -1190707246000], [2510579891232450, 4364291343719], [-5600068878825156, -3638555441450], [334038659834850, -5863392431019], [5773382304604706, -353876140150], [8972419403090900, -4222403268602], [-13199835885524270, -2275409211625], [-6093557090054932, 433131013325], [-1246059743213284, -1479876203100], [-5583964082179450, 3505048730350], [-5522086730172800, 605520844086], [-3737665132030650, -7974218077996], [2436609439990850, 4425545561566], [22306478033043700, 1657990887530], [8751170397408928, -91272013400], [-11779022184961250, -3470145662968], [-4594307207520270, -7756452899800], [-13384980695437412, -4174620266050], [-8761171219881384, 3107480695600], [11991952700019716, 4097368116050], [-20978019196497600, -6803407957964], [-7761027678539700, -3072974520259], [-4494862387029990, -4188982039050], [4562021592069050, 9902803169072], [15773619421286018, -3628466041575], [-1610610571683804, 8200793505400], [-7423883440576350, -16228072817258], [-2509837500812796, -4667053738625], [9355940803360100, 1487903927022], [-37596048180627350, 7271377590566], [-37023934208803368, 8434924337400], [-28883871378875900, 1826048531745], [977161304715750, 4989935936856], [34819850238953794, 1679269334100], [50039055972177050, -5291320097359], [55212564477709500, -95355965628], [-11464765034351778, 1245360860100], [-18203262786065800, -5647523284900], [24577599932561050, -12649868402240], [10789921139977700, 131200200109], [22012979071482986, 11730356465600], [2481779988148300, -5969731657184], [16385607922956282, -281110423075], [-40437020686799452, 14945546082175], [15519053700047470, -15856222617300], [-41821022622992636, 13141285123625], [-24853505013046900, -5878645972574], [-17289108477192050, -20086063937118], [2822061338841850, -34212495220016], [-354908907406032, -12584225674000], [74725609707987674, 16009752731225], [-48226768010087850, 22151880693954], [-55613768626946252, 25774427164200], [46334214158147450, 5381890629900], [36602604347155200, -1848256694352], [10314105005260450, -43089927804488], [-9651668432596708, 16479829283600], [-26864978671173500, -22121760933648], [57177093479796290, -27276992871275], [12957555462020884, -22571154831600], [-28613784964047750, 50835468985496], [-7211973464565470, 68782197572200], [-107956265387626250, -31544024983454], [-115528739841617150, -30433814587720], [64651579602342232, 56703182574300], [59657092229775234, 3071792246825], [-39721909977203300, 1955784158427], [-11057200630232050, 5372153869382], [-71949341410939566, 4442773912200], [-148215894613215100, 4410013411558], [-24880853217985350, 80762348552491], [-50477103535639300, -66307359526041], [-58299677526911252, -2356717497950], [-177096541547939446, 10916821817300], [-90552809197015100, 55892721013282], [-129819890987369950, 35556278450512], [12192115082068890, -2708804574175], [62188313969777652, 20706683103975], [-139740452563069716, -60270955816800], [1954931953050400, -54281279084170], [-5270786474152710, -48650786645400], [41849117322551956, 38352625176225], [206557159763667098, 27188246996650], [78623543478238136, -81757738359000], [-37200237537516750, -94429745458167], [-3807928438014370, 59695556176350], [-52729317189219500, 501073138023], [146838123002015706, 44758008917750], [-797837985825028, -25076381828925], [125753758181415804, 84710521629050], [110108181588391910, 93021236377500], [154602899654631450, 95868810878020], [-402786693801675000, 8103258821914], [330806761094597684, -12516968154525], [-92574021769707700, 55195580698492], [-25470268452084650, -64242113762882], [274945037520042700, 34199915003498], [-51817095297562900, 173522364986723], [-153336449460514470, -35534769271400], [227373776309736388, -142997810294350], [-258737526706302184, 133414956486450], [-124618953043475230, 29441919120900], [26127597853768650, 103287445700333], [-152288685957170700, -73900494374642], [-121564603719710800, 92297849550834], [-145965612302683908, 59124012061850], [252052892397143100, 102796480323068], [-40983267770019252, -144147866892975], [131377380010543566, -24059723644700], [-11822845126883950, -30927379021344], [-113371337161613200, 144670912671566], [-102338395264838284, -160173927589475], [-106293058718699100, -104440617501140], [-116590291528623700, 62748181996172], [57944696226806272, -12944590516700], [-313009279213755450, 67052181031294], [-20161966780437998, 282356161239100], [-175495469089064976, 110819959792250], [-7442453327102790, 134047774663400], [-166116911269166150, 100560195005807], [348587661568214100, 133211353058092], [60676603369651278, -39315036628650], [-437781013824320644, -62153197702900], [-456781869920289150, 59307077013815], [-429184711957409750, 190240830297160], [37512399224194268, 221557482640575], [188021034535579764, -373974956236200], [9208538391866174, -201603000254850], [156764159875316350, 87665175356804], [262624577545988250, 63032008714206], [173659830171829700, -63637672474636], [-267510567279329326, -175939818873675], [-180304746196923878, -273082852589500], [-525512627295669308, 83998087731150], [-616038649159237350, -52789014299635], [256017421233137342, 235303285339800], [-372154778562090604, -102438653601100], [304267871777794250, 87410660856288], [142355603694140100, -143006340249455], [-1050897714997035950, 41628279132384], [547207227684345588, -168339614626225], [-168875302172652364, 618565692932750], [441748460608852814, -137876214301200], [-54148899103446710, -271655334537200], [-115226900583800300, 670033871714636], [900385843601885448, -198210434878150], [-476675909708755100, -415303077077377], [-69339857059048496, 48363733780150], [652875570318722598, -190071279027000], [-71976086940416716, -258577388216250], [968261908539115200, -183926284629716], [365188412441828050, -162201272869148], [-260362629547665350, 572362219884681], [-243250180515220900, -370803942045423], [769108370383414130, 258261712775100], [510494580177434610, -574407073821275], [378411769072350084, 290321261063600], [641112051698633850, -547133098658044], [298941698351518300, -144310959049040], [116043548253910350, 993362321620380], [1031692572854951730, 475154184979025], [210588206116168700, 324861708629391], [-1539513963331357950, -193042960655133], [-26412370433568300, -722270103007642], [-981111315318604724, 457829013086350], [-302388164556672950, 536789654690208], [-624705804221048750, -316416127011147], [-600313974195679100, 427075926684805], [233650575390894308, -484714646904200], [-110628816735963100, -441554830205914], [-829905332008637150, -465330311764772], [-147800206455873324, -944743218500750], [-212953792371360954, -218829083858350], [100703360801895600, -629114967352362], [-959349001186593850, -609754440680832], [-1222630718722122672, 89143460436200], [481141835836430114, -701069841946525], [399872481832508458, 398991670626850], [821139659045571068, -409764180246350], [868592491526499096, -41241502494300], [1426224249439946074, -942070619506200], [-1993867941839059500, 643980249490012], [158785992314876600, 270291353606062], [334243709636926450, -1101408771456136], [1627179978563691330, -284370137819100], [730692930338561700, 1248130485836200], [-1408639225334066950, 252314507980992], [2672998235482702570, -15608021247825], [148585874965250132, -365975525324875], [-872055792491542350, -19843010447266], [-1692203138637265310, 322404302655100], [363970891364910300, -1380519793186238], [978557802924183492, 161744225303550], [-639065909736990496, 401929407287600], [1403077221856175450, 513437729745245], [-134470897313166410, -1339422659687850], [1557559448380990450, -609449434834879], [-934787266815062900, -716069038855781], [-4007387217484090268, 45901160298100], [1349407803209570300, 261530069552184], [182693809114068404, -985474363705450], [1174989589473520830, -151322637642300], [1569379533868267250, -1245586615668220], [1636286939124042890, -1236025779696400], [3145717736057900076, -663238217229225], [585914483869821350, -170819578386076], [-1948819168156589950, 356224987110718], [3351213280523826290, 116808807054875], [-911229803371153900, 2366747000404931], [2780207493662741050, -479517082346989], [4573654456613911500, -186843108838814], [2481289945108251150, 414122725262250], [115032906644958316, 1254687109509100], [581879336849422600, -263781869481530], [-873263125073125550, 1084209561165935], [-1348889473471090100, 114970582773998], [138587539794665450, -2990521307503300], [3875748215398957130, 525577851712675], [4261083540839833932, -712693249610025], [913006283889979974, 1931524164008500], [-2447359063892750550, 1498476655644456], [5674821997131882000, -285465181947628], [2155389125272411500, 1504128410388260], [-5007830702028002150, 218605446732578], [600113230948299848, -876387876654100], [-5968906577393573450, 923008269070004], [-3923527117615905078, -847996414300100], [5218334898440887184, 70267105106200], [-3476060746028673100, -130043220252570], [-1112512201413805756, 1587619568909900], [7362404856084433450, -553177294798880], [-5273980642048784550, 143100727037047], [-3596484318241229188, 1310783322667725], [-3405517441370059670, -122249905688800], [-6719593601275776900, 884088023910070], [-4151743595834943100, -605650036304606], [-1714587031685823408, -707881694349400], [2462434734521097570, -2102854630550925], [-908052802843291300, -1616465273903527], [-5814272841651630250, 487683259475770], [-1719671481437772900, -2137673389816406], [-506675854137968800, -4960134196218568], [-5443533241978597150, -865255906217472], [823340858445488490, -1615710115374650], [-4238928959691429750, -307599125316700], [-1185163046373126318, 3518216664103425], [-3315548532426931550, 1646912857250522], [-509517093681889156, 2447672464387625], [-2274064632397197150, 357292920352464], [-2713123477828535300, 2848090973818938], [5398600471347446888, -2088460575680400], [3401439478465334100, 2882443548320641], [3237685399092217550, 3198876527969542], [-2413045950713587508, 1164028602098800], [3770972822882922824, 3261593333960650], [8436859794537135650, -2317157831864200], [3036872389521972650, -3172442560531047], [6294085775749971900, -2005263695414192], [2274584529892454476, -340325692647350], [-2429868496337308600, -1214016017338340], [5330714849324929050, -1057262967905700], [-3491405229955878500, -3982632820467577], [5858862149649440466, 1355059611121550], [-10003870426178635900, -245541977601646], [-15771375833972150, 4144623825406125], [8489968329858346990, -2502265630343800], [9880676804532349050, 292943052057694], [-3794060575615638356, 3576737787154575], [-5671212044012997150, -3986982554838442], [-9646525862675304910, -666685522598325], [-10558862549339246500, 106309269746573], [-2629702281330409450, 3757353461516638], [12515380618402071082, 1041107343202750], [-386510438175175668, 3676647155254850], [-493082602189274900, 906284176695138], [-1010811458634492350, -2452874010526600], [-12620844496693799750, 2034453635895065], [12459468828357681828, -2065736620208150], [-2326953669564304500, -4718994372043682], [-9038694790441426550, -1595700831696464], [9535801741181098298, -4436629864935425], [-1865596476888097600, 5830368297656360], [-12321263584721945764, 2114345292657925], [2840010016232811050, -7328821488007466], [5535851666882361808, 5834906629646550], [2615128541066554450, -3457049722445775], [299196355762894550, 5456272828083484], [-14701179185129190176, -2409215234141300], [-8983861411492965900, 1580526629070006], [-3274514753147936950, -2939978760335264], [-5381574094509654950, -4032406342242257], [-1183668149385305652, 382472124543350], [-2847069362198326310, -6669213702881500], [671113072376804500, 5185608554717778], [3185601343146768000, -5707885681131822]]
